/*
 * PM - 2017
 * lab0.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "usart.h"
#include "dht.h"


#define __STOP__ 0x00
#define __START__ 0x78
#define __REQUEST__TEMP__ 0x33
#define __SCHEME_SHIFT__ 0x09
#define __CHECK_STATUS__ 0x06

#define __MAX_LENGTH__ 30
#define __MAX_LEVEL__ 4
#define __MAX_SCHEMES__ 4

int8_t state = __STOP__;
int8_t command;
int8_t temp;
int8_t humidity;
int8_t level;
int8_t executing = 0;


char temp_msg[__MAX_LENGTH__];
char debug_msg[__MAX_LENGTH__];
int8_t scheme_level[__MAX_LEVEL__];


ISR(USART1_RX_vect)
{


	command = UDR1;

	switch(command)
	{
   		case __REQUEST__TEMP__:
   			dht_gettemperaturehumidity(&temp, &humidity);
   			memset(temp_msg, 0, __MAX_LENGTH__);
   			sprintf(temp_msg, "Temperature: %d\nHumidity: %d\n",
   				temp, humidity);
   			USART1_print(temp_msg);
   			break;
   		case __SCHEME_SHIFT__:

   			scheme_level[level] = (scheme_level[level] + 1) % __MAX_SCHEMES__;
   			break;

   		case __CHECK_STATUS__:
   			memset(debug_msg, 0, __MAX_LENGTH__);
   			sprintf(debug_msg, "state %d\n, executing %d\n",
   					 state, executing);
   			USART1_print(debug_msg);
   			break;


   		case __START__:
   			state = __START__;
   			break;

   		case __STOP__:
   			state = __STOP__;
   			break;
	}


}

void realise_scheme(int8_t scheme_code)
{

	switch (scheme_code) {

		case 0:
			PORTA = 0;
			PORTA |= (1 << PA0) | (1 << PA1);

			_delay_ms(2);

			PORTA = 0;
			PORTA |= (1 << PA1) | (1 << PA2);

			_delay_ms(2);
			PORTA = 0;
			PORTA |= (1 << PA2) | (1 << PA3);

			_delay_ms(2);
			PORTA = 0;
			PORTA |= (1 << PA3) | (1 << PA0);

			_delay_ms(2);

			break;

		case 1:
			PORTA = 0;
			PORTA |= (1 << PA1);

			_delay_ms(2);

			PORTA = 0;
			PORTA |= (1 << PA2);

			_delay_ms(2);
			PORTA = 0;
			PORTA |= (1 << PA3);

			_delay_ms(2);
			PORTA = 0;
			PORTA |= (1 << PA0);

			_delay_ms(2);

			break;

		case 2:

			PORTA = 0;
			PORTA |= (1 << PA3);

			_delay_ms(2);

			PORTA = 0;
			PORTA |= (1 << PA2);

			_delay_ms(2);
			PORTA = 0;
			PORTA |= (1 << PA1);

			_delay_ms(2);
			PORTA = 0;
			PORTA |= (1 << PA0);

			_delay_ms(2);

			break;

		case 3:

			PORTA = 0;
			PORTA |= (1 << PA3) | (1 << PA0);

			_delay_ms(2);

			PORTA = 0;
			PORTA |= (1 << PA2) | (1 << PA3);

			_delay_ms(2);
			PORTA = 0;
			PORTA |= (1 << PA1) | (1 << PA2);

			_delay_ms(2);
			PORTA = 0;
			PORTA |= (1 << PA0) | (1 << PA1);

			_delay_ms(2);

			break;




	}
}


void init(void)
{

	DDRB |= (1 << PB0) | (1 << PB2);

	DDRD &= ~(1 << PD0);

	DDRA |= (1 << PA0) | (1 << PA1) | (1 << PA2) | (1 << PA3);

	DDRA &= ~(1 << PA4);

	int i;
	for (i = 0; i < __MAX_LEVEL__; i++) {
		scheme_level[i] = 0;
	}
	PORTB = 0;

	USART1_init();

}

void determine_level(void)
{

		dht_gettemperaturehumidity(&temp, &humidity);
		if (temp < 10) {
			PORTB = 0;
			PORTB |= (1 << PB0);
			level = 0;
		} else if (temp >= 10 && temp < 20) {
			PORTB = 0;
			PORTB |= (1 << PB1);
			level = 1;
		} else if (temp >= 20 && temp < 30) {
			PORTB = 0;
			PORTB |= (1 << PB2);
			level = 2;
		} else {
			PORTB = 0;
			PORTB |= (1 << PB3);
			level = 3;
		}
}

void rotate(void){

	int rotations = 0;
	determine_level();


	while(1){


		if( ((PIND & (1 << PD0)) == 1 ) && (state == __START__))
		{

			realise_scheme(scheme_level[level]);
			rotations++;

		}
		if (rotations == 30000) {
			rotations = 0;
			determine_level();
		}
	}
}


int main(void) {

	sei();
	init();
	rotate();

	while(1);

	return 0;
}
